#!/usr/bin/env python
#
# Copyright 2005,2006,2007,2009 Free Software Foundation, Inc.
# 
# This file is part of GNU Radio
# 
# GNU Radio is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# GNU Radio is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with GNU Radio; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

from gnuradio import gr, gru, modulation_utils2
from gnuradio import usrp
from gnuradio import eng_notation
from gnuradio.eng_option import eng_option
from optparse import OptionParser

import random
import struct
import sys
import time 

# from current dir
import uhd_receive_path

#import os
#print os.getpid()
#raw_input('Attach and press enter: ')

class my_top_block(gr.top_block):
    def __init__(self, demodulator, rx_callback, options):
        gr.top_block.__init__(self)

        # Set up receive path
        self.rxpath = uhd_receive_path.uhd_receive_path(demodulator, rx_callback, options) 

        self.connect(self.rxpath)


class cs_mac(object):

    def __init__(self):
        self.tb = None
        self.n_rcvd0 = 0
        self.n_right0 = 0
        self.init_time = time.time()

    def set_top_block(self, tb):
        self.tb = tb


    def rx_callback(self, ok, payload):
        
        (pktno,) = struct.unpack('!H', payload[0:2])
        self.n_rcvd0 += 1
        if ok:
            self.n_right0 += 1

        print "ok = %5s  pktno = %4d  n_rcvd0 = %4d  n_right0 = %4d" % (
            ok, pktno, self.n_rcvd0, self.n_right0)


        if ( (self.init_time + 60)>time.time() ) or( self.n_rcvd0 == 1200):
            self.tb.stop()


# /////////////////////////////////////////////////////////////////////////////
#                                   main
# /////////////////////////////////////////////////////////////////////////////

def main():

    demods = modulation_utils2.type_1_demods()

    # Create Options Parser:
    parser = OptionParser (option_class=eng_option, conflict_handler="resolve")
    expert_grp = parser.add_option_group("Expert")

    parser.add_option("-m", "--modulation", type="choice", choices=demods.keys(), 
                      default='dbpsk2',
                      help="Select modulation from: %s [default=%%default]"
                            % (', '.join(demods.keys()),))

    uhd_receive_path.add_options(parser, expert_grp)

    for mod in demods.values():
        mod.add_options(expert_grp)

    (options, args) = parser.parse_args ()

    if len(args) != 0:
        parser.print_help(sys.stderr)
        sys.exit(1)

    if options.rx_freq is None:
        sys.stderr.write("You must specify -f FREQ or --freq FREQ\n")
        parser.print_help(sys.stderr)
        sys.exit(1)

    mac=cs_mac()

    # build the graph
    tb = my_top_block(demods[options.modulation], mac.rx_callback, options)

    mac.set_top_block(tb)

    r = gr.enable_realtime_scheduling()
    if r != gr.RT_OK:
        print "Warning: Failed to enable realtime scheduling."

    init_time = time.time()
    tb.start()        # start flow graph
    
    while 1:
        #Xprint "loop"
        if (init_time + 20)>time.time():
            tb.stop()
    
    tb.wait()         # wait for it to finish

if __name__ == '__main__':
    try:
        main()
    except KeyboardInterrupt:
        pass
